/* 
 HP203B Pressure & Temperature Sensor Library
 By: Andrew Shelestov
 Date: April 19th, 2017
 License: This code is public domain but you buy me a beer if you use this and we meet someday (Beerware license).
 
 Get pressure and temperature from the HP203 sensor.
 */

#include <Wire.h>

#include "HP203B.h"

HP203B::HP203B() {
  m_addr = HP203_DEFAULT_ADDR;
}

HP203B::HP203B(int addr) {
  m_addr = addr;
}

void HP203B::begin(){
  Wire.begin();
}

int HP203B::getcp(float* p_tempC,  float* p_pressure) {
  float cTemp, pressure;
  unsigned int data[6] = {0};

  if(NULL == p_tempC  && NULL == p_pressure){
    return -1;
  }

  // Start I2C transmission
  Wire.beginTransmission(m_addr);
  // Send OSR and channel setting command
  Wire.write(0x40 |0x04 | 0x00);
  // Stop I2C transmission
  Wire.endTransmission();
  delay(500);

  // Start I2C transmission
  Wire.beginTransmission(m_addr);
  // Select data register
  Wire.write(0x10);
  // Stop I2C transmission
  Wire.endTransmission();

  // Request 6 bytes of data
  Wire.requestFrom(m_addr, 6);

  // Read 6 bytes of data
  // cTemp msb, cTemp csb, cTemp lsb, pressure msb, pressure csb, pressure lsb
  if (Wire.available() == 6) {
    data[0] = Wire.read();
    data[1] = Wire.read();
    data[2] = Wire.read();
    data[3] = Wire.read();
    data[4] = Wire.read();
    data[5] = Wire.read();
  }
  if(NULL == data[0] && NULL == data[1] && NULL == data[2] && NULL == data[3] && NULL == data[4] && NULL == data[5]){
    return -1;
  }

  // Convert the data to 20-bits
  cTemp = (((data[0] & 0x0F) * 65536) + (data[1] * 256) + data[2]) / 100.00;
  pressure = (((data[3] & 0x0F) * 65536) + (data[4] * 256) + data[5]);

  if(NULL != p_tempC){
    *p_tempC = cTemp;
  }

  if(NULL != p_pressure){
    *p_pressure = pressure;
  }
  return 0;
}


float HP203B::readTemperature() {
  float tempC;

  getcp(&tempC,  NULL);

 return tempC;
}

float HP203B::readPressure() {
  float pressure;

  getcp( NULL, &pressure);

 return pressure;
}

float HP203B::readAltitude() {

  float altitude;
  unsigned int data[3] = {0};

  // Start I2C transmission
  Wire.beginTransmission(m_addr);
  // Send OSR and channel setting command
  Wire.write(0x40 |0x04 | 0x00);
  // Stop I2C transmission
  Wire.endTransmission();
  delay(500);

  // Start I2C transmission
  Wire.beginTransmission(m_addr);
  // Select data register
  Wire.write(0x31);
  // Stop I2C transmission
  Wire.endTransmission();

  // Request 6 bytes of data
  Wire.requestFrom(m_addr, 3);

  // Read 3 bytes of data
  // cTemp msb, cTemp csb, cTemp lsb, pressure msb, pressure csb, pressure lsb
  if (Wire.available() == 3) {
    data[0] = Wire.read();
    data[1] = Wire.read();
    data[2] = Wire.read();

  }
  if(NULL == data[0] && NULL == data[1] && NULL == data[2] /*&& NULL == data[3] && NULL == data[4] && NULL == data[5]*/){
    return -1;
  }

  altitude= (((data[0] & 0x0F) * 65536) + (data[1] * 256) + data[2]) / 100.00;

return altitude;
}